# Copyright Collab 2015

# shortcuts
from .util import extract_tower_json

# version information
__version__ = (1, 0, 0, 'b2')

#: For example: `2.0.0`
short_version = '.'.join([str(x) for x in __version__[:3]])

#: For example: `2.0.0a1`
version = '{}{}'.format('.'.join([str(x) for x in __version__[:-1]]),
    __version__[-1])



# Copyright Collab 2015

"""
Extract and compile localization strings for the require.js i18n plugin.
"""

import os
from optparse import make_option

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand, CommandError
from django.utils.translation import get_language_info, check_for_language

from require_i18n.core import UpdateCatalog


class Command(BaseCommand):
    help = 'Compile gettext catalogs used in the require.js i18n plugin.'

    option_list = BaseCommand.option_list + (
        make_option("-l", "--locale",
            dest="locale",
            help="Locale(s) to process (e.g. nl). Default is to process" +
                " all. Can be used multiple times.",
            default="all",
            metavar="LOCALE"
        ),
        make_option("-d", "--domain",
            dest="domain",
            help="The domain of the message files. Default: %default",
            default="site",
            metavar="DOMAIN"
        ),
        make_option("--dry-run",
            action="store_true",
            dest="dry_run",
            default=False,
            help="Simulate without updating files. Default: %default"
        ),
        make_option("--no-empty",
            action="store_true",
            dest="no_empty",
            default=False,
            help="Empty compiled translation files are not allowed and will " +
                "be deleted if detected. Default: %default"
        ),
        make_option("-t", "--output-type",
            dest="output_type",
            help="The type for the translated output files, either 'json' " +
                "for JSON format or 'script' for Javascript files. " +
                "Default: %default",
            default="script",
            metavar="TYPE"
        )
    )

    def handle(self, *args, **options):
        self.locale_dir = settings.LOCALE_PATHS[0]

        # parse options
        self.dry_run = options['dry_run']
        self.domain = options['domain']
        self.no_empty = options['no_empty']
        self.output_type = options['output_type']

        # parse languages
        self.languages = []
        locale = options['locale']
        if locale == 'all':
            self.languages = getattr(settings, 'LANGUAGES', [])
        else:
            if check_for_language(locale):
                info = get_language_info(locale)
                self.languages.append((info['code'], info['name']))
            else:
                raise CommandError('Not a valid locale: {}'.format(locale))

        self.output_dir = os.path.join(settings.ROOT, 'locale', 'templates',
            'LC_MESSAGES')

        self.catalog_update = UpdateCatalog(self.output_dir, self.locale_dir,
            self.domain, self.languages, self.dry_run, self.no_empty,
            self.output_type)
        self.catalog_update.stdout = self.stdout

        # extract js translations
        self._extract()

        # compile js translations
        self._compile()

    def _extract(self):
        """
        Extract translations.
        """
        # update extraction
        try:
            call_command('extract', interactive=False, dry_run=self.dry_run,
                verbosity=0, outputdir=self.output_dir, domain=self.domain,
                create=True)
        except KeyError:
            pass

        self.stdout.write('\n')
        self.stdout.write('Updating catalogs for domain {}...'.format(
            self.domain))

        self.catalog_update.extract()

    def _compile(self):
        """
        Compile translations.
        """
        self.stdout.write('\n')
        self.stdout.write(
            'Updating javascript translations for domain {}...'.format(
            self.domain))

        self.catalog_update.compile()



# Copyright Collab 2015

import os
import glob
import codecs
from json import dumps, loads
from datetime import datetime

from django.conf import settings
from django.core.management.base import CommandError

import polib

from babel.dates import format_datetime


#: License header for catalogs
DEFAULT_HEADER = """Copyright (C) My Project
This file is distributed under the same license as the Foo project.
"""
REQUIRE_I18N_HEADER = getattr(settings, 'REQUIRE_I18N_HEADER',
    DEFAULT_HEADER)

#: Javascript template for parsing and writing files
DEFAULT_JS_TEMPLATE = """// Copyright (C) My Project

define(
{0}
);
"""
REQUIRE_I18N_JS_TEMPLATE = getattr(settings, 'REQUIRE_I18N_JS_TEMPLATE',
    DEFAULT_JS_TEMPLATE)


class UpdateCatalog(object):
    """
    Extract and compile Javascript translation files for the requirejs i18n
    plugin.
    """
    def __init__(self, output_dir, locale_dir, domain, languages,
                 dry_run=False, no_empty=False, output_type='script'):
        self.output_dir = output_dir
        self.locale_dir = locale_dir
        self.domain = domain
        self.languages = languages
        self.dry_run = dry_run
        self.no_empty = no_empty
        self.output_type = output_type

    def extract(self):
        """
        Extract translations.
        """
        # extract for each language
        for lang in self.languages:
            code, label = lang

            self.input_pot_path = os.path.join(self.output_dir,
                self.domain + '.pot')
            self.update_po_path = os.path.join(self.locale_dir,
                code.replace('-', '_'), 'LC_MESSAGES', self.domain + '.po')

            if os.path.exists(self.update_po_path):
                # updating existing po
                update_po = polib.pofile(self.update_po_path)
            else:
                # create new po
                update_po = self.addLanguage(self.update_po_path, code, label)

            if not os.path.exists(self.input_pot_path):
                raise CommandError('Input catalog not found: {}'.format(
                    self.input_pot_path))

            # merge extraction data
            extracted_pot = polib.pofile(self.input_pot_path)
            update_po.merge(extracted_pot)

            # update revision date
            self.updateMeta(update_po, code, label)
            self.updateDate(update_po, ['PO-Revision-Date'])

            # add copyright header
            update_po.header = REQUIRE_I18N_HEADER

            if self.dry_run is False:
                # save
                update_po.save(self.update_po_path)

            self.stdout.write('  {} ({}): {}% translated.'.format(code, label,
                update_po.percent_translated()))

    def compile(self):
        """
        Compile translations.
        """
        for section in settings.DOMAIN_METHODS.get(self.domain):
            path, formatter = section
            jobs = []

            # scan for javascript translation source files
            for js_file in glob.glob(os.path.join(settings.ROOT, path)):

                # load javascript source file
                with open(js_file, 'r') as f:
                    # parse source file for json data
                    inputData = ''
                    lines = f.readlines()
                    for line in lines[3:-1]:
                        inputData += line.rstrip()

                    # load json data
                    outputData = loads(inputData, encoding='utf8')

                    # i18n translations root path, eg. static/js/site/nls
                    out_dir = os.path.normpath(os.path.join(
                        os.path.dirname(js_file), ".."))

                    # scan locale input dirs for .po files,
                    # eg. locale/nl/LC_MESSAGES/site.po
                    for lang in self.languages:
                        code, label = lang
                        locale = code.replace('-', '_')

                        # the .po file, eg. locale/nl/LC_MESSAGES/site.po
                        po_path = os.path.join(self.locale_dir, locale,
                            'LC_MESSAGES', self.domain + '.po')

                        # make sure .po exists
                        if os.path.exists(po_path):
                            # create output dir for files,
                            # eg. static/js/site/nls/nl
                            out_path = os.path.join(out_dir, locale)
                            if not os.path.exists(out_path) and (
                                self.dry_run is False):
                                os.makedirs(out_path)

                            # create job for output js file,
                            # eg. static/js/site/nls/nl/account.js
                            if self.output_type == "json":
                                # use JSON extension: .json
                                fname, ext = os.path.splitext(os.path.basename(
                                    js_file))
                                outfile_path = os.path.join(out_path,
                                    fname + ".json")
                            else:
                                # use script extension: .js
                                outfile_path = os.path.join(out_path,
                                    os.path.basename(js_file))
                            jobs.append((locale, po_path, outfile_path,
                                outputData, js_file))

            # compile new javascript translation files
            for locale, po_path, out_path, json_data, js_file in jobs:

                self.stdout.write('  {}'.format(out_path))

                translation = self.po2dict(po_path, json_data)

                # get file contents
                fileContent = self.render_output(translation)

                if self.dry_run is False:
                    # nothing to translate and we want to ignore the file
                    if len(translation.keys()) == 0 and self.no_empty is True:
                        # file already exists because it previously contained
                        # translations
                        if os.path.exists(out_path):
                            # remove file
                            os.remove(out_path)
                    else:
                        # write js translation file
                        with codecs.open(out_path, 'w', encoding='utf8') as writer:
                            writer.write(fileContent)

    def updateDate(self, po, fields):
        """
        Update the catalog's date fields in metadata.
        """
        for field in fields:
            if field in po.metadata:
                po.metadata[field] = format_datetime(
                    datetime.now(),
                    'yyyy-MM-dd HH:mmZ',
                    locale='en')

    def updateMeta(self, po, code, label):
        """
        """
        po.metadata = {
            'Project-Id-Version': '1.0',
            'Report-Msgid-Bugs-To': 'i18n-bugs@example.com',
            'POT-Creation-Date': '2015-02-03 01:17+0100',
            'PO-Revision-Date': '2015-02-03 02:17+0100',
            'Last-Translator': 'Your Name <you@example.com>',
            'Language-Team': '{label} <{code}@example.com>',
            'Language': "{code}",
            'MIME-Version': '1.0',
            'Content-Type': 'text/plain; charset=utf-8',
            'Content-Transfer-Encoding': '8bit',
        }
        po.metadata.update(settings.REQUIRE_I18N_PO_METADATA)

        if 'Language-Team' in po.metadata:
            po.metadata['Language-Team'] = po.metadata['Language-Team'].format(
                label=label, code=code)
        if 'Language' in po.metadata:
            po.metadata['Language'] = po.metadata['Language'].format(
                code=code)

    def addLanguage(self, path, code, label):
        """
        Create new catalog for a language.
        """
        if self.dry_run is False:
            # create directory for .po file
            if os.path.exists(os.path.dirname(self.update_po_path)) is False:
                os.makedirs(os.path.dirname(self.update_po_path))

        po = polib.POFile()

        self.updateMeta(po, code, label)

        # set date fields to current time
        self.updateDate(po, ['POT-Creation-Date'])

        return po

    def po2dict(self, po_path, json_data):
        """
        Compare catalog and json data and translate entries.
        """
        catalog = polib.pofile(po_path)

        for key, val in json_data.items():
            # find translation entry in catalog
            entry = catalog.find(val)

            if entry and entry.translated():
                # save translation
                json_data[key] = entry.msgstr
            else:
                # get rid of this item, we don't have a translation for it
                del json_data[key]

        return json_data

    def render_output(self, structure, indent=1, **k):
        """
        Render output.

        It also escapes the forward slash, making the result suitable
        to be included in an HTML <script> tag.
        """
        result = dumps(structure, indent=indent, **k).replace('/', '\/')

        if self.output_type is None or self.output_type == "script":
            return REQUIRE_I18N_JS_TEMPLATE.format(result)

        elif self.output_type == "json":
            return result



# Copyright Collab 2015

"""
URLConf for :py:mod:`require_i18n` tests.
"""

from django.contrib import admin
from django.conf.urls import patterns, include, url


admin.autodiscover()

urlpatterns = patterns('',
    url(r'^', include(admin.site.urls)),
)



# Copyright Collab 2015

"""
Utilities.
"""

from tower import tweak_message

from pybabel_json.extractor import extract_json


def extract_tower_json(fileobj, keywords, comment_tags, options):
    """
    JSON Babel extractor.
    """
    for lineno, funcname, message, comments in (
            list(extract_json(fileobj, keywords, comment_tags, options))):

        message = tweak_message(message)

        yield lineno, funcname, message, comments



# Copyright Collab 2015

"""
Tests for the :py:mod:`require_i18n.management.commands` package.
"""

import os
import json
from glob import glob
from shutil import rmtree

from django.test import TestCase
from django.conf import settings
from django.core.management import call_command
from django.core.management.base import CommandError

import polib


class CompileJSTestCase(TestCase):
    """
    Tests for the :py:mod:`require_i18n.management.commands.compile_js`
    command.
    """
    locale_path = settings.LOCALE_PATHS[0]
    cmd_name = "compile_js"

    bad_locale = "frysk"
    good_locale = "nl"
    new_locale = "fr"

    bad_domain = "pan"
    good_domain = "test"
    default_domain = "site"

    def setUp(self):
        self._cleanTestFiles()

    def tearDown(self):
        self._cleanTestFiles()

    def _cleanTestFiles(self, enabled=True):
        """
        """
        # remove .po and .pot
        if enabled is True:
            files = []
            for root, dirnames, filenames in os.walk(self.locale_path):
                files.extend(glob(os.path.join(root, "{}.po*".format(
                             self.good_domain))))

            for f in files:
                if os.path.exists(f):
                    os.remove(f)

    def test_missingInputCatalog(self):
        """
        A `CommandError` is raised when the input catalog can not be found.
        """
        with self.assertRaises(CommandError) as cm:
            call_command(self.cmd_name, interactive=False, dry_run=True,
                domain=self.bad_domain)

        self.assertEqual(str(cm.exception),
            'Input catalog not found: {}/templates/LC_MESSAGES/{}.pot'.format(
            self.locale_path, self.bad_domain))

    def test_invalidLocale(self):
        """
        A `CommandError` is raised when the locale option contains an
        unknown locale code.
        """
        with self.assertRaises(CommandError) as cm:
            call_command(self.cmd_name, interactive=False, dry_run=True,
                locale=self.bad_locale)

        self.assertEqual(str(cm.exception), 'Not a valid locale: {}'.format(
            self.bad_locale))

    def test_validLocaleDefaultDomain(self):
        """
        A .po file is created for a valid locale with the default domain.
        """
        call_command(self.cmd_name, interactive=False,
            locale=self.good_locale, no_empty=True)

        po_file_path = os.path.join(self.locale_path, self.good_locale,
            "LC_MESSAGES", self.default_domain + ".po")

        # the .po file exists
        self.assertTrue(os.path.exists(po_file_path))

        #  there should be no empty translation files when the
        # --no-empty option is enabled
        self.assertFalse(os.path.exists(os.path.join(settings.ROOT,
            "static", "js", self.default_domain, "nls", self.good_locale,
            "foo.js")))

        # custom copyright header should be present
        po_file = polib.pofile(po_file_path)
        self.assertEqual(po_file.header, settings.REQUIRE_I18N_HEADER)

    def test_validLocaleCustomDomain(self):
        """
        A .po file is created for a valid locale with a custom domain.
        """
        call_command(self.cmd_name, interactive=False,
            locale=self.good_locale, domain=self.good_domain,
            no_empty=True)

        pot_file_path = os.path.join(self.locale_path, 'templates',
            "LC_MESSAGES", "{}.pot".format(self.good_domain))
        po_file_path = os.path.join(self.locale_path, self.good_locale,
            "LC_MESSAGES", "{}.po".format(self.good_domain))

        # the .po and .pot files exist
        self.assertTrue(os.path.exists(pot_file_path))
        self.assertTrue(os.path.exists(po_file_path))

        # custom copyright header should be present
        po_file = polib.pofile(po_file_path)
        self.assertEqual(po_file.header, settings.REQUIRE_I18N_HEADER)

        # custom metadata is present
        self.assertEqual(po_file.metadata['Report-Msgid-Bugs-To'],
            'i18n-bugs@root')
        self.assertEqual(po_file.metadata['Language'], self.good_locale)
        self.assertEqual(po_file.metadata['Language-Team'],
            'Dutch <{}@root>'.format(self.good_locale))

        # translate and save the file
        for entry in po_file:
            if entry.msgid == "red":
                entry.msgstr = "rood"
            elif entry.msgid == "yellow":
                entry.msgstr = "geel"
        po_file.save(po_file_path)

        # run the command again
        call_command(self.cmd_name, interactive=False,
            locale=self.good_locale, domain=self.good_domain,
            no_empty=True)

        # the translated file exists
        trans_path = os.path.join(settings.ROOT,
            "static", "js", self.good_domain, "nls", self.good_locale,
            "colors.js")
        self.assertTrue(os.path.exists(trans_path),
            "Translated file expected, does not exist: {}".format(
            trans_path))

        # remove translations
        for entry in po_file:
            if entry.msgid == "red":
                entry.msgstr = ""
            elif entry.msgid == "yellow":
                entry.msgstr = ""
        po_file.save(po_file_path)

        # run the command again
        call_command(self.cmd_name, interactive=False,
            locale=self.good_locale, domain=self.good_domain,
            no_empty=True)

        # the translated file is gone because no_empty is enabled
        self.assertFalse(os.path.exists(trans_path))

        # remove translated output file and the directory it's in
        rmtree(os.path.dirname(trans_path))

    def test_newLocaleCustomDomain(self):
        """
        A .po file is created for a new locale with a custom domain.
        """
        call_command(self.cmd_name, interactive=False,
            locale=self.new_locale, domain=self.good_domain,
            no_empty=True)

        po_file_path = os.path.join(self.locale_path, self.new_locale,
            "LC_MESSAGES", "{}.po".format(self.good_domain))

        # the new .po file exists
        self.assertTrue(os.path.exists(po_file_path))

        # remove .po
        rmtree(os.path.join(self.locale_path, self.new_locale))

        # remove nls directory
        rmtree(os.path.join(settings.ROOT, "static", "js", self.good_domain,
            "nls", self.new_locale))

    def test_outputTypeJson(self):
        """
        """
        call_command(self.cmd_name, interactive=False,
            locale=self.good_locale, domain=self.good_domain,
            no_empty=True, output_type="json")

        po_file_path = os.path.join(self.locale_path, self.good_locale,
            "LC_MESSAGES", "{}.po".format(self.good_domain))
        po_file = polib.pofile(po_file_path)

        # translate and save the file
        for entry in po_file:
            if entry.msgid == "red":
                entry.msgstr = "rood"
            elif entry.msgid == "yellow":
                entry.msgstr = "geel"
        po_file.save(po_file_path)

        # run the command again
        call_command(self.cmd_name, interactive=False,
            locale=self.good_locale, domain=self.good_domain,
            no_empty=True, output_type="json")

        # the translated file exists
        trans_path = os.path.join(settings.ROOT,
            "static", "js", self.good_domain, "nls", self.good_locale,
            "colors.json")
        self.assertTrue(os.path.exists(trans_path),
            "Translated file expected, does not exist: {}".format(
            trans_path))

        # translation is correct
        with open(trans_path, 'r') as p:
            expectedResult = {u'redColor': u'rood', u'yellowColor': u'geel'}
            translation = json.loads(p.read())
            self.assertDictEqual(expectedResult, translation)



# Copyright Collab 2015

import os

SITE_ID = 1

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

#: Local time zone for this installation. Choices can be found here:
#: http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
#: although not all choices may be available on all operating systems.
#: In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'Europe/Amsterdam'

#: Customized copyright header
REQUIRE_I18N_HEADER = """Copyright (C) 2015 Collab
This file is distributed under the same license as the django-require-i18n project.
"""

#: Customized Javascript template
REQUIRE_I18N_JS_TEMPLATE = """// Copyright Collab 2015

define(
{0}
);
"""

#: Custom metdata for .po file
REQUIRE_I18N_PO_METADATA = {
    'Project-Id-Version': '1.0',
    'Report-Msgid-Bugs-To': 'i18n-bugs@root',
    'Last-Translator': 'Foo <you@root>',
    'Language-Team': '{label} <{code}@root>'
}

# Tower root
ROOT = os.path.dirname(__file__)

#: dict of domain to file spec and extraction method tuples.
DOMAIN_METHODS = {
    'site': [
        ('static/js/site/nls/root/*.js', 'require_i18n.extract_tower_json'),
    ],
    'test': [
        ('static/js/test/nls/root/*.js', 'require_i18n.extract_tower_json'),
    ]
}

TEXT_DOMAIN = 'messages'

#: Jinja configuration for tower.
def JINJA_CONFIG():
    config = {'extensions': ['tower.template.i18n',
                             'jinja2.ext.with_',
                             'jinja2.ext.loopcontrols'],
              'finalize': lambda x: x if x is not None else ''}
    return config

# function that takes arbitrary set of args and combines them with ROOT to
# form a new path.
path = lambda *args: os.path.abspath(os.path.join(ROOT, *args))

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.sessions',
    'django.contrib.contenttypes',
    'django.contrib.admin',
    'tower',
    'require_i18n',
    'require_i18n.tests'
]

ROOT_URLCONF = 'require_i18n.tests.urls'

SECRET_KEY = 'top_secret'

STATIC_URL = '/static/'

# A tuple of directories where Django looks for translation files.
LOCALE_PATHS = (
    os.path.join(ROOT, "locale"),
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

# Logging configuration.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.

LOG_HANDLER = 'console'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'formatters': {
        'verbose': {
            'format': '%(asctime)s %(levelname)-6s %(name)-15s - %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(asctime)s %(levelname)-6s %(name)-15s - %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple',
        },
        'null': {
            'level': 'DEBUG',
            'class': 'django.utils.log.NullHandler',
        }
    },
    'loggers': {
        'require_i18n': {
            'handlers': [LOG_HANDLER],
            'level': 'DEBUG'
        },
    }
}



# Copyright Collab 2015

from setuptools import setup

from require_i18n import version


INSTALL_REQUIRES = [
    "django",
    "Babel",
    "polib>=1.0.6",
    "pybabel-json>=0.1.0",
    "tower>=0.4.1"
]


setup(
    name = "django-require-i18n",
    version = version,
    license = "MIT",
    description = "Extract and compile localization strings for the " +
        "require.js i18n plugin.",
    author = "Thijs Triemstra",
    author_email = "info@collab.nl",
    url = "https://github.com/collab-project/django-require-i18n",
    install_requires=INSTALL_REQUIRES,
    packages = [
        "require_i18n",
        "require_i18n.management",
        "require_i18n.management.commands",
    ],
    keywords='django requirejs i18n plugin require.js',
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Internationalization",
        "Topic :: Software Development :: Localization",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
    ],
)



# Copyright Collab 2015

"""
Tests for the :py:mod:`require_i18n.util` module.
"""

from io import BytesIO

from django.conf import settings
from django.test import TestCase

from babel.messages.extract import DEFAULT_KEYWORDS, extract

from tower.management.commands.extract import (create_pofile_from_babel,
    OPTIONS_MAP, COMMENT_TAGS)


TOWER_KEYWORDS = dict(DEFAULT_KEYWORDS)

if hasattr(settings, 'TOWER_KEYWORDS'):
    TOWER_KEYWORDS.update(settings.TOWER_KEYWORDS)


def fake_extract_from_dir(filename, fileobj, method, options=OPTIONS_MAP,
                          keywords=TOWER_KEYWORDS, comment_tags=COMMENT_TAGS):
    """
    We use Babel's extract_from_dir() to pull out our gettext
    strings. In the tests, there's no directory of files, only BytesIO
    objects. So, we fake the original function with this one.
    """
    for lineno, message, comments, foo in extract(method, fileobj, keywords,
            comment_tags, options):
        yield filename, lineno, message, comments


class UtilTestCase(TestCase):

    def test_extract_tower_json(self):
        """
        Tests for :py:func:`require_i18n.util.extract_tower_json`.
        """
        po_input = """
        define({
            "root": {
                "cannot_load_bg": "Error for you!",
            }
        });
        """
        po_output = """\
#: filename:4
msgid "Error for you!"
msgstr ""
"""
        fileobj = BytesIO(po_input)
        method = 'require_i18n.extract_tower_json'

        output = fake_extract_from_dir('filename', fileobj, method,
            keywords=TOWER_KEYWORDS, options=OPTIONS_MAP,
            comment_tags=COMMENT_TAGS)
        self.assertEqual(po_output, unicode(create_pofile_from_babel(output)))
